<?php

namespace App\Util;

use App\Exception;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = []) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = [];
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }
        
        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function writeLog($type, $fileName, $content) {
        $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config = json_decode($configFile, true);
        
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";
        
        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = $folder . $fileName;
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = $folder . $fileName;
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = $folder . $fileName;
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = $folder . "$fileName.json";
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            
            file_put_contents($currentFile, $content);
        }
        
        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }
    }

    public function sendMail($recipient, $cc, $subject, $params)
    {
        try
        {
            $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config     = json_decode($configFile, true);
            $body       = file_get_contents(__DIR__ . '/../../templates/defaultMail.html');
            
            $mail = new PHPMailer(true);
            
            $mail->isSMTP();                            // Set mailer to use SMTP
            $mail->Host       = $config["host"];        // Specify main and backup SMTP servers
            $mail->SMTPAuth   = true;                   // Enable SMTP authentication
            $mail->Username   = $config["username"];    // SMTP username
            $mail->Password   = $this->decrypt($config["password"]);    // SMTP password
            $mail->SMTPSecure = $config["SMTPSecure"];  // Enable TLS encryption, `ssl` also accepted
            $mail->Port       = $config["port"];        // TCP port to connect to
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["from"], "Teknisa Retail");
            
            foreach ($recipient as $r) {
                $mail->addAddress($r);
            }
            
            if(empty($cc)) {
                $cc = ["suporte.odhen@teknisa.com"];
            }
            
            foreach ($cc as $c) {
                $mail->addCC($c);
            }
            
            $mail->isHTML(true);
            $mail->Subject = "$subject ";
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("SMTP error: " . $e->getMessage());
        }
    }
    
    public function getPendingOrdersFile() {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder = $basePath."/dispatch";

        $fileName = "/pendingOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }
        
        if(!file_exists($currentFile)){
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function getTogoOrdersFile() {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        $basePath     = $config['logPath'] . "/$clientFolder";
        $folder       = $basePath."/dispatch";
        $fileName     = "/togoOrders.json";
        $currentFile  = $folder . $fileName;
        
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }
        
        return $currentFile;
    }
    
    public function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = is_array($networkData['header']) ? implode($networkData['header'], "\r\n") : $networkData['header'];
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $body      = is_array($networkData['body']) ? implode($networkData['body'], "\r\n") : $networkData['body'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];
        
        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':BODY'      => $body,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];
        
        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }
    
    /*
     * Função que tranforma uma string de um valor decimal em
     * seu respectivo valor no tipo float
    */
    public function tofloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
        
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        }
        
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        return  str_replace(".", ",", $tmp);
    }
    
    /*
     * Função para buscar email dentre uma lista de emails parametrizados pelo
     * Cliente - Email para receber detalhamentos de Erro
    */
    public function getMailList() {
        $list = [];
        
        $sql = "
            SELECT DISTINCT DSEMAILTEC
              FROM LISTAEMAILTEC
              WHERE (CDPROJETO = '011' OR CDPROJETO = '003')
              ORDER BY DSEMAILTEC
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }
    
    /*
     * Função que busca dentro do json de TimeZones do projeto
     * o estado do cliente para buscar o Default Time Zone correto para o php
    */
    public function getCurrentTimezone ($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Sao_Paulo",    "ES" => "America/Sao_Paulo",
            "GO" => "America/Sao_Paulo",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Sao_Paulo",    "PR" => "America/Sao_Paulo",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Sao_Paulo",    "RN" => "America/Fortaleza",
            "RS" => "America/Sao_Paulo",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Sao_Paulo",
            "SE" => "America/Maceio",       "SP" => "America/Sao_Paulo",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }

    /*
     * Função que remove acentos da string
    */
    public function removeAccents($text) {
        $table = ['á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_'];
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($text, $table));        
    }

    /*
     * Função para formatar numeros de ponto flutuante 
     * para acompanhar o dbSchema
    */
    public function formatNumber($number) {
        return str_replace(",", ".", $number);
    }
    
    /*
     * Função para decodificar dados sensiveis do Cliente
    */
    private function decrypt($string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
        
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
        
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);
        
        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    }
    
    public function getExecutionLog(){
        $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config = json_decode($configFile, true);
        
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";
        $folder       = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
        $currentFile  = "$folder" . date("d") . ".txt";
        
        $content = "";
        
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
            
        }
        if(!file_exists($currentFile)) {
            $content = date("d/m/Y H:i:s") . ": [   Integridade  ] Verificação de integridade.\n";
        }
        file_put_contents($currentFile, $content, FILE_APPEND);
        
        return $currentFile;
    }
    
    public function formatNrSeqProdCom($seq) {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }
    
    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
}
